<script setup>
import {$go} from "../../composables/goto";
import {reactive, ref} from "vue";

let statuses = ref()


</script>
<template>
    <AppLayout title="Clients">
        <template #header>
            <h2 class="font-semibold text-xl text-gray-800 leading-tight"> Licenses
                <el-button text type="primary" size="small" class="ml-4" rounded @click="$go(route('licenses.create'))">
                    Add License
                </el-button>
            </h2>
        </template>
        <div>
            <div class="max-w-6xl mx-auto py-5 sm:px-2 mt-8 ">
                <div class="flex bg-white px-3 py-3 border-b-2 !rounded-b-none">
                    <div class="flex">
                        <i class="mr-4" v-if="filtered">{{ filters }}</i>
                        <div class="mr-4" v-else>
                            Filter:
                        </div>
                        <el-check-tag type="success" effect="dark" @click="clearFilters" round>Clear Filters</el-check-tag>
                    </div>
                </div>
                <el-empty description="No license has been added yet." v-if="!licenses.length">
                    <el-button @click="$go(route('licenses.create'), {data: {}})" link class="uppercase" size="small"
                               type="primary">
                        Add License
                    </el-button>
                </el-empty>
                <div v-else>
                    <el-table ref="licensesTable" size="small" :data="licenses" style="width: 100%">
                        <el-table-column label="#" width="25">
                            <template #default="scope">
                                {{ scope.$index + 1 }}
                            </template>
                        </el-table-column>
                        <el-table-column prop="client.name" label="Organisation" max-width="130px"
                                         :filters="clientFilters" :filter-method="filterClients" :filtered-value="filteredClients">
                            <template #default="scope">
                                {{ scope.row.client.name }} <i class="text-blue-500">({{ scope.row.client?.username }})</i>
                            </template>
                        </el-table-column>
                        <el-table-column prop="status" label="Status" align="center" width="100px"
                            :filters="statusFilters" :filter-method="filterStatus">
                            <template #default="scope">
                                <el-tag size="small" effect="light" :type="statusType(scope.row.status)">
                                    {{ scope.row.status }}
                                </el-tag>
                            </template>
                        </el-table-column>
                        <el-table-column prop="project.name" label="Project" show-overflow-tooltip align="center" width="100px">
                            <template #default="scope">
                                <strong class="text-green-500">{{ scope.row.project.name }}</strong>
                            </template>
                        </el-table-column>
                        <el-table-column prop="updated_at" label="Lastly updated" show-overflow-tooltip>
                            <template #default="scope">
                                {{ $date(scope.row.updated_at)}}
                            </template>
                        </el-table-column>
                        <el-table-column prop="user.name" label="User" show-overflow-tooltip align="center" width="100px"/>
                        <el-table-column prop="start_date" label="Start" align="center" width="100px"/>
                        <el-table-column prop="end_date" label="End" align="center" width="100px"/>
                        <el-table-column align="center" prop="user.id" label="Duration" width="80px">
                            <template #default="scope">
                                {{ $date(scope.row.end_date).diff(scope.row.start_date, 'M') }} M
                            </template>
                        </el-table-column>
                        <el-table-column label="Actions" min-width="50px" class="grid grid-cols-2 gap-4 place-content-start">
                            <template #header>
                                <el-icon size="small" class="el-icon--left">
                                    <Tools/>
                                </el-icon>
                                Actions
                            </template>
                            <template #default="scope">
                                <ConfirmsPassword @confirmed="updateStatus(scope.row.id, scope.$index)">
                                    <el-button :type="canChangeStatus(scope.row.status_code) ? statusType(nextStatus(scope.row.status)):'primary'"
                                               v-if="scope.row.status_code < 1" size="small">
                                        <el-icon class="el-icon--left">
                                            <ArrowRightBold/>
                                        </el-icon> ACTIVATE
                                    </el-button>
                                </ConfirmsPassword>
                                <el-button @click="revokeLicense(scope.row.id, scope.$index)" type="danger"
                                           size="small" class="ml-1" :disabled="!canRevoke(scope.row.status_code)">
                                    <el-icon class="el-icon--left">
                                        <Delete/>
                                    </el-icon>
                                    Revoke
                                </el-button>
                                <el-dropdown class="float-right mt-1">
                                    <span class="el-dropdown-link">
                                      <el-icon class="el-icon--right rotate-90"><More/></el-icon>
                                    </span>
                                    <template #dropdown>
                                        <el-dropdown-menu>
                                            <el-dropdown-item :disabled="!canMail(scope.row.status_code)"
                                                              :icon="Message">E-Mail License</el-dropdown-item>
                                            <el-dropdown-item v-if="canRefresh(scope.row.status_code)" :icon="Refresh">
                                                Renew License (New)</el-dropdown-item>
                                            <el-dropdown-item @click="copyText(scope.row.token)" :icon="CopyDocument">Copy Token</el-dropdown-item>
                                        </el-dropdown-menu>
                                    </template>
                                </el-dropdown>
                            </template>
                        </el-table-column>
                    </el-table>
                </div>
            </div>
        </div>
    </AppLayout>
</template>

<script>
import AppLayout from "@/Layouts/AppLayout.vue";
import ConfirmsPassword from "@/Components/ConfirmsPassword.vue";
import {Tools, Tickets, ArrowRightBold, Delete, Message, Refresh, More, CopyDocument} from "@element-plus/icons-vue";
import {reactive} from "vue";
import {copyTextToClipboard} from "../../composables/clipboard";

export default {
    name: "Licenses",
    components: {AppLayout, Tools, Tickets, Message, Delete, Refresh, More, ArrowRightBold, ConfirmsPassword},
    data() {
        return {
            licenses: this.$page.props.licenses,
            statuses: {PENDING: 'info', ACTIVE: 'success', EXPIRED: 'warning', REVOKED: 'danger'},
            query: {
                status: null,
                search: '',
            },
            filteredClients: []
        }
    },
    computed: {
        statusFilters () {
            return [
                {text: 'ACTIVE', value: 1},
                {text: 'EXPIRED', value: 2},
                {text: 'PENDING', value: 0},
                {text: 'REVOKED', value: -1},
            ];
        },
        clientFilters () {
            let clients = []
            this.licenses.map(l => {
                if (!clients.find(cc => cc.value === l.client.id)) {
                    clients.push({text: l.client.name, value: l.client.id})
                }
            })
            return clients
        },
        filtered () {
            return Object.keys(this.$page.props.filters).length
        },
        filters () {
            let str = ""
            let filters = this.$page.props.filters
            Object.keys(filters).forEach((key, index) => {
                if (key === 'client') {
                    str += `Client: ${this.licenses.find(l => l.client_id === parseInt(filters[key])).client.name}`
                }
            })

            return str
        },
        filteredByStatus () {
            return !this.query.status ? this.licenses : this.licenses.filter(l => l.status_code === this.query.status)
        },
        filteredByName () {
            return !this.query.search ? this.licenses : this.licenses.filter(l => l.name)
        }
    },
    methods: {
        clearFilters () {
            if (this.filtered) $go(route('licenses'))
            else this.$refs.licensesTable.clearFilter()
        },
        filterStatus (code, row) {
            return row.status_code === code
        },
        filterClients (id, row) {
            return row.client_id === id
        },
        canChangeStatus(code) {
            return code === 0
        },
        statusType(status) {
            return this.statuses[status]
        },
        nextStatus(status) {
            let statuses = Object.keys(this.statuses)
            return statuses[statuses.indexOf(status) + 1]
        },
        canMail(code) {
            return code >= 1
        },
        canRevoke(code) {
            return code >= 0 && code <= 1
        },
        canRefresh(code) {
            return code >= 2
        },
        copyText(text) {
            if (text) {
                copyTextToClipboard(text)
                this.$notify.success('Secret token successfully copied to clipboard')
            }
        },
        updateStatus (license, index) {
            axios.post(route('licenses.status', license))
                .then(res => res.data)
                .then(data => {
                    this.licenses.splice(index, 1, data)
                })
        },
        revokeLicense (license, index) {
            axios.post(route('licenses.revoke', license))
                .then(res => res.data)
                .then(data => {
                    this.licenses.splice(index, 1, data)
                })
        }
    }
}
</script>

<style scoped>

</style>
